class Edit:
    """
    Klasse for endring av observasjoner. Instanseres med n-gramsrrelse
    og treningsdata.
    """
    def __init__(self, n=3, s=4):
        self.n = n # n-gramstrrelse
        self.s = s # suffikslengde (lengste suffiks som skal telles)

    def add_data(self, trainingdata, M):
        """
        Tar en liste med annoterte setninger og en modell M, legger til dataene i M
        """
        self.data = trainingdata
        self.model = M
        self.words = {} # ord/tagg-forekomster
        self.ngrams = {} # ngrammer
        self.suffixes = {} # suffiks/tagg-forekomster
        self.sentences = len(self.data) # antall setninger som skal legges til
        self.iterate_corpus() # g gjennom nye setninger og tell opp
        #for hvert element i data, ngrams, words suffixes og sentences, legg til i M
        self.add_ngrams()
        self.add_words()
        self.add_suffixes()

    def del_data(self, trainingdata, M):
        """
        Tar en liste med annoterte setninger og fjerner de fra treningsdata
        """
        self.data = trainingdata
        self.model = M
        self.words = {} # ord/tagg-forekomster
        self.ngrams = {} #ngrammer
        self.suffixes = {} # suffiks/tagg-forekomster
        self.sentences = len(self.data) # antall setninger som skal fjernes
        self.iterate_corpus()
        # tilsvarende som over, men subtraksjon i stedet for addisjon
        self.del_ngrams()
        self.del_words()
        self.del_suffixes()


#############################################################
## Funskjoner for  legge til eller fjerne data fra modellen
#############################################################

    def add_ngrams(self):
        """
        Legger til nye ngrammer i modellen
        """
        for nsize in self.ngrams: # for hver ngramstrrelse
            for ngram in self.ngrams[nsize]: # for hver ngram av gitt strrelse
                if ngram in self.model.ngrams[nsize]: #hvis ngrammen er observert fr
                    self.model.ngrams[nsize][ngram] = self.model.ngrams[nsize][ngram] + self.ngrams[nsize][ngram] # oppdater antall
                else: # hvis ngrammen ikke er observert fr
                    self.model.ngrams[nsize][ngram] = self.ngrams[nsize][ngram] # legg til antall
                
    def del_ngrams(self):
        """
        Fjerner ngrammer fra modellen
        """
        for nsize in self.ngrams: # for hver ngramstrrelse
            for ngram in self.ngrams[nsize]: # for hver ngram gitt strrelse
                if ngram in self.model.ngrams[nsize]: # hvis ngrammen er observert fr
                    self.model.ngrams[nsize][ngram] = self.model.ngrams[nsize][ngram] - self.ngrams[nsize][ngram]
                    if self.model.ngrams[nsize][ngram] < 1: # hvis antall av ngrammen er mindre enn 1...
                        del self.model.ngrams[nsize][ngram] # fjern ngrammen fra modellen
                        
    def add_words(self):
        """
        Legger til ord i modellen
        """
        for word in self.words: # for hvert ord som skal legges til
            if word in self.model.words: # hvis ordet er sett fr
                for tag in self.words[word]:
                    if tag in self.model.words[word]: # hvis taggen er observert med ordet fra fr     
                        self.model.words[word][tag] = self.model.words[word][tag] + self.words[word][tag] # oppdater antall
                    else: # ordet er sett fr, men ikke med taggen
                        self.model.words[word][tag] = self.words[word][tag] # legg til antall for taggen gitt ordet
            else: # hvis ordet er nytt i modellen
                self.model.words[word] = {} # opprett dict for ordet
                for tag in self.words[word]: 
                    self.model.words[word][tag] = self.words[word][tag] # legg til antall for taggen gitt ordet

    def del_words(self):
        """
        Fjerner ord fra modellen
        """
        for word in self.words: # for hvert ord som skal fjernes
            if word in self.model.words: # hvis ordet er sett fr
                for tag in self.words[word]:
                    if tag in self.model.words[word]: # hvis taggen er observert med ordet fra fr     
                        self.model.words[word][tag] = self.model.words[word][tag] - self.words[word][tag] # oppdater antall
                        if self.model.words[word][tag] < 1: # hvis ord/tagg-korrelasjonen er sett mindre enn 1 ganger...
                            del self.model.words[word][tag] # fjern taggen
                            if self.model.words[word] == {}: # hvis ordets dict er tom (jmf forrige linje)
                                del self.model.words[word] # fjern ordet

    def add_suffixes(self):
        """
        Legger til suffikser i modellen
        """
        for suffix in self.suffixes: # for hvert suffiks som skal legges til
            if suffix in self.model.suffixes: # hvis suffikset er sett fr
                for tag in self.suffixes[suffix]:
                    if tag in self.model.suffixes[suffix]: # hvis taggen er observert med suffikset fra fr
                        self.model.suffixes[suffix][tag] = self.model.suffixes[suffix][tag] + self.suffixes[suffix][tag] # oppdater antall
                    else: # suffikset er sett fr, men ikke med taggen
                        self.model.suffixes[suffix][tag] = self.suffixes[suffix][tag] # legg til antall for taggen gitt ordet
            else: # hvis suffikset er nytt i modellen
                self.model.suffixes[suffix] = {} # opprett dict for suffikset
                for tag in self.suffixes[suffix]: 
                    self.model.suffixes[suffix][tag] = self.suffixes[suffix][tag] # legg til antall for taggen gitt suffikset
        
    def del_suffixes(self):
        """
        Fjerner suffikser fra modellen
        """
        for suffix in self.suffixes: # for hvert suffiks som skal legges til
            if suffix in self.model.suffixes: # hvis suffikset er sett fr
                for tag in self.suffixes[suffix]:
                    if tag in self.model.suffixes[suffix]: # hvis taggen er observert med suffikset fra fr
                        self.model.suffixes[suffix][tag] = self.model.suffixes[suffix][tag] - self.suffixes[suffix][tag] # oppdater antall
                        if self.model.suffixes[suffix][tag] < 1: # hvis suffiks/tagg-korrelasjonen er sett mindre enn 1 ganger...
                            del self.model.suffixes[suffix][tag] # fjern taggen
                            if self.model.suffixes[suffix] == {}: # hvis suffiksets dict er tom (jmf forrige linje)
                                del self.model.suffixes[suffix] # fjern suffikset


######################################################################################################
## Funksjoner for opptelling av ngrammer, ord og suffikser for data som skal legges til eller fjernes
######################################################################################################       

    def iterate_corpus(self):
        """
        Gr gjennom treningskorpus og lagrer n-grammer og tagg/ord-korrelasjoner
        """
        # nb! endret i forhold til opptreningsalgoritmen
        self.init_ngrams()
        import operator
        counter = 0
        for line in self.data: # for hver setning
            L = line.split()
            self.count_ngrams(line) # tell n-grammer
            self.count_words(L) # tell ord og suffikser
            counter = counter + 1
            if operator.mod(counter,1000) == 0:
                print str(counter) + " linjer lest..."

    def init_ngrams(self):
        """
        Initialiserer n-gramtabellen
        """
        self.ngrams[0] = {() : 0} # totalt antall tagger
        for i in range(self.n):
            self.ngrams[i+1] = {}

    def count_ngrams(self, line):
        """
        Tar en setning som argument. Teller ngrammer (over tagger) og lagrer i ngramtabellen.
        """
        import re
        T2 = []
        t = re.compile(r'<(.+?):')
        T = t.findall(line) # plukk ut taggene, lagre i T
        for temptag in T: # for hver forelpige tagg
            t2 = re.compile(r'.+(\/|\|)(.*)') # let etter / eller |
            tag = t2.search(temptag) 
            if tag:
                T2.append(tag.group(2))
            else:
                T2.append(temptag)
        self.ngrams[0][()] = self.ngrams[0][()] + len(T) # oppdater totalt antall tagger
        for i in range(self.n): # for hver aktuelle ngramstrrelse
            self.count_ngram(T2, i+1) # teller ngrammer av strrelse i+1

    def count_ngram(self, T, nsize):
        """
        Tar ei liste med tagger (T) og en ngramstrrelse (nsize) som argumenter.
        Lagrer alle ngrammer av strrelse nsize.
        """
        length = len(T)
        for i in range(nsize - 1): 
            T.insert(0, None) # sett inn n-1 "dummy"-symboler
        for i in range(length): # for hver tagg
            ngram = [] # initialiser ngram
            for j in range(nsize): # for 0 til n-1
                ngram.append(T[i+j]) # legg til tagg til ngrammen
            ngram = tuple(ngram) # endre fra liste til tuppel
            if ngram in self.ngrams[nsize]: # hvis ngrammen er observert fr
                self.ngrams[nsize][ngram] = self.ngrams[nsize][ngram] + 1 # oppdater antall
            else: # hvis ikke ngrammen er observert fr
                self.ngrams[nsize][ngram] = 1 # sett antall lik 1
        
    def count_words(self, L):
        """
        Tar en setning som argument (som liste). Teller ord/tagg-forekomster og
        suffiks/tagg-forekomster og lagrer i dictionaries
        """
        import re
        for wordtag in L: # for hvert ord/tagg-par
            w = re.compile(r'^(.+?)<')
            t = re.compile(r'<(.+?):')
            word = w.search(wordtag) # finn ordet
            tag = t.search(wordtag) # finn taggen
            tagg = tag.group(1)
            self.count_word(word.group(1), tagg) # lagre ord/tagg-relasjon
            self.count_suffixes(word.group(1), tagg) # lagre suffiks/tagg-relasjoner

    def count_word(self, word, tag):
        """
        Tar et ord og en tagg som argumenter. Lagrer ordet og taggen i ordtabellen.
        """
        if word in self.words: # hvis ordet er sett fr
            if tag in self.words[word]: # hvis ordet er sett med den aktuelle taggen fr
                self.words[word][tag] = self.words[word][tag] + 1 # oppdater antall
            else:
                self.words[word][tag] = 1 # hvis ordet er sett fr, men ikke med den aktuelle taggen
        else:
            self.words[word] = {tag:1} # hvis ikke ordet er sett fr
        
    def count_suffixes(self, word, tag):
        """
        Tar et ord og en tagg som argumenter, lagrer ordets suffikser sammen med taggen i suffikstabellen
        """
        for i in range(1, self.s): # fra 1 til maks suffikslengde - 1...
            if len(word) > i:  #hvis ordet er minst like langt som den aktuelle suffikslengden
                suffix = word[-(i+1):] # plukk ut suffiks
                self.count_suffix(suffix,tag) # lagre suffiks/tagg-relasjon

    def count_suffix(self, suffix, tag):
        """
        Tar et suffiks og en tagg som argumenter, lagrer suffikset med taggen i suffikstabellen
        """
        if suffix in self.suffixes: # hvis suffikset er sett fr
            if tag in self.suffixes[suffix]: # hvis suffikset er sett med den aktuelle taggen
                self.suffixes[suffix][tag] = self.suffixes[suffix][tag] + 1 # oppdater antall
            else:
                self.suffixes[suffix][tag] = 1 # hvis suffikset er sett fr, men ikke med den aktuelle taggen
        else:
            self.suffixes[suffix] = {tag:1} # hvis ikke suffikset er sett fr


#######################################
## Klasse for utskrift av data til fil
#######################################
            
class Print:
    """
    Klasse for skriving av antall til fil
    """
    def __init__(self, P):
        self.P = P # instans av klassen Count

    def print_ngrams(self):
        """
        Skriver ngrammer til fil
        """
        for i in self.P.ngrams:
            if i > 0:
                ngramfile = open(str(i)+ "_edit.txt", 'w')
                for j in self.P.ngrams[i]:
                    ngramfile.write(str(j) + "\t" + str(self.P.ngrams[i][j]) + "\n")
                ngramfile.close()
                
    def print_words(self):
        """
        Skriver ord til fil
        """
        wordfile = open("words_edit.txt", 'w')
        for word in self.P.words:
            for tag in self.P.words[word]:
                wordfile.write(str(word) + "\t" + str(tag) + "\t" + str(self.P.words[word][tag]) + "\n")
        wordfile.close()

    def print_suffixes(self):
        """
        Skriver suffikser til fil
        """
        suffixfile = open("suffixes_edit.txt", 'w')
        for suffix in self.P.suffixes:
            for tag in self.P.suffixes[suffix]:
                suffixfile.write(str(suffix) + "\t" + str(tag) + "\t" + str(self.P.suffixes[suffix][tag]) + "\n")
        suffixfile.close()

    def print_to_files(self):
        """
        Skriver data til fil
        """
        print "Skriver til fil..." + "\n"
        self.print_ngrams()
        self.print_words()
        self.print_suffixes()
