"""
Finner interpolerte ngram-verdier. (parameter A)
Finner kandidatsett for bde kjente og ukjente ord (parameter B)
- interpolasjon/vekter
ukjente ord:
    - suffiksvekter
    - tall
    - pne ordklasser
"""

class Model:
    """
    Klasse for modellen.
    """

    def __init__(self,Data):
        self.weights = [[0.9, 0.1], [0.9, 0.09, 0.01], [0.9, 0.09, 0.009, 0.001]] # vekter for interpolering
        self.suffixweights = [0.9, 0.09, 0.01] # vekter for suffiksdistribusjon
        self.ngrams = Data.ngrams
        self.words = Data.words
        self.suffixes = Data.suffixes
        self.n = Data.n # ngramstrrelse
        self.total = Data.total # totalt antall tagger/tokens
        self.OpenTags = ['tall', 'noun', 'noun_prop', 'verb', 'adj']
        self.Num = ['tall'] # liste med tallord-tagger, m tilpasses taggsettet
        self.Proper = ['noun_prop'] # liste med egennavn-tagger, m tilpasses taggsettet
        ## Disse br flyttes!!!:
        self.Numbers = [] # variabel for ukjente ord som har blitt klassifisert som tall
        self.ProperNouns = [] # variabel for ukjente ord som har blitt klassifisert som egennavn,
        self.Compounds = [] # variabel for ukjente ord som har blitt klassifisert som sammensetning
        self.UnknownSuffix = [] # variabel for ukjente ord som har ftt taggdistribusjon basert p suffikser
        self.KnownSuffix = [] # variabel for ukjente ord der ingen suffikser er sett fr


    def clear_lists(self):
        """
        Sletter lister som er spesifikke for hver setning
        """
        self.Numbers = [] 
        self.ProperNouns = [] 
        self.Compounds = [] 
        self.UnknownSuffix = [] 
        self.KnownSuffix = [] 

        
    def get_trans_prob(self,ngram):
        """
        Finner interpolert verdi for en ngram
        """
        import math
        w = self.weights[len(ngram)-2] # plukk ut vekter
        prob = 0
        i = 0
        if ngram == (None,):
            prob = 1.0
        else:
            for j in range(len(ngram), 0, -1): # for hver ngramstrrelse (3, 2 og 1)
                if ngram[i:] in self.ngrams[j]:
                    prob = prob + ((float(self.ngrams[j][ngram[i:]]) / self.ngrams[j-1][ngram[i+1:]]) * w[i])
                i = i+1
        return math.log(prob) # returnerer logaritmen av sannsynligheten


    def get_cand(self,word,First=1):
        """
        Finner kandidatsettet til et ord
        """
        import math
        C = {}
        if word in self.words:  # hvis kjent ord
            for tag in self.words[word]:
                C[tag] = math.log(float(self.words[word][tag]) / self.ngrams[1][tuple([tag])])
        else: # hvis ukjent ord
            C = self.get_unknown_cand(word,First) # finn kandidatsett
        return C # returner kandidatsett


    def get_unknown_cand(self,word,First=1):
        """
        Finner kandidatsett for et ukjent ord
        (First = 0 betyr at man ser p frste ord i en setning...)
        """
        import math
        import re
        num = re.compile(r'^[0-9].*[0-9]$')
        number = num.search(word)
        if number: # hvis tallord
            self.Numbers.append(word)
            Temp = {}
            for numbertag in self.Num:
                Temp[numbertag] = 0
            return Temp
        else:
            if First > 0: # hvis ikke frste ord
                sta = re.compile(r'^[A-Z]')
                start = sta.search(word)
                if start: # hvis stor forbokstav
                    self.ProperNouns.append(word)
                    Temp = {}
                    for propertag in self.Proper:
                        Temp[propertag] = 0
                    return Temp
            if len(word) > 3:
                compound = self.find_compound(word)
                if compound:
                    self.Compounds.append(word)
                    return self.get_cand(compound)
            return self.suffix_prob(word)

    def find_compound(self,word):
        """
        Tar inn et ord, og avgjr om det er en sammensetning
        Hvis det er en sammensetning returneres siste ord i sammensetningen,
        hvis ikke returneres tom liste. Gjelder kun for norsk, ikke engelsk.
        """
	for i in range(len(word), 3, -1):
            if word[-i:] in self.words and (word[:-i] in self.words or \
                                            (word[:-i-1] in self.words and \
                                             (word[len(word)-i-1] == "s" or word[len(word)-i-1] == "e"))):
                return word[-i:]
            # kan eventuelt interpolere distribusjonene til alle mulige sammensetninger
        return [] # hvis ingen sammensetning er funnet

    def suffix_prob(self,word):
        """
        Returnerer taggdistribusjonen for et ukjent ord gitt ordets suffikser.
        Begrenser mulige tagger til de pne ordklassene
        """
        import math
        c1 = {} # interpolert kandidatsett for suffikser av lengde 2 - 4
        c2 = {} # kandidatsett for suffiks av lengde 2
        c3 = {} # kandidatsett for suffiks av lengde 3
        c4 = {} # kandidatsett for suffiks av lengde 4
        if len(word) > 5: # hvis ordet har minst 6 bokstaver
            suf = word[-4:] # suffiks av lengde 4
            if suf in self.suffixes:
                for tag in self.suffixes[suf]:
                    c4[tag] = float(self.suffixes[suf][tag]) / self.ngrams[1][tuple([tag])]
        if len(word) > 4: # hvis ordet har minst 5 bokstaver
            suf = word[-3:] # suffiks av lengde 3
            if suf in self.suffixes:
                for tag in self.suffixes[suf]:
                    c3[tag] = float(self.suffixes[suf][tag]) / self.ngrams[1][tuple([tag])]
        if len(word) > 3: # hvis ordet har minst 4 bokstaver
            suf = word[-2:] # suffiks av lengde 2
            if suf in self.suffixes:
                for tag in self.suffixes[suf]:
                    c2[tag] = float(self.suffixes[suf][tag]) / self.ngrams[1][tuple([tag])]
                self.KnownSuffix.append(word)
                c1 = self.known_suffixes(c2, c3, c4)
        if c1 == {}: # unntakshndtering hvis ingen av de andre metodene gir resultater
            c1 = self.unknown_suffixes()
        return c1

    def unknown_suffixes(self):
        """
        Returnerer jevn sannsynlighetsdistribusjon for alle pne tagger.
        Ndlsning! (har ingen klare holdepunkter for hvilken tagg man skal velge,
        ngram-modellen avgjr
        """
        c = {}
        for tag in self.OpenTags:
            c[tag] = 0
        return c
            
    def known_suffixes(self, c2, c3, c4):
        """
        Returnerer interpolert sannsynlighetsdistribusjon for tagger basert px de tre distribusjonene
        c2, c3 og c4
        """
        c = {}
        for tag in self.OpenTags:
            if tag in c4:
                c[tag] = c4[tag] * self.suffixweights[0]
            if tag in c3:
                if tag in c:
                    c[tag] = c[tag] + (c3[tag] * self.suffixweights[1])
                else:
                    c[tag] = c3[tag] * self.suffixweights[1]
            if tag in c2:
                if tag in c:
                    c[tag] = c[tag] + (c2[tag] * self.suffixweights[2])
                else:
                    c[tag] = c2[tag] * self.suffixweights[2]
        # gjr om til logaritmer
        import math
        for tag in c:
            c[tag] = math.log(c[tag])
        return c

