"""
gaa gjennom korpus
ta n som input
tell n-grammer
tell ord/tagg forekomster
beregn sannsynligheter

datastruktur:
dictionary med n som index og n-gramtabell som verdi
"""

class Count:
    """
    Klasse for telling av observasjoner. Instanseres med n-gramstorrelse
    og korpusfilnavn.
    """
    def __init__(self, trainingcorpus, n=3, s=4):
        self.n = n # n-gramstorrelse
        self.s = s # suffikslengde (lengste suffiks som skal telles)
        self.data = trainingcorpus # treningskorpus (filnavn)
        self.ngrams = {} # n-gramtabell
        self.words = {} # ord/tagg-forekomster
        self.suffixes = {} # suffiks/tagg-forekomster
        self.sentences = 0 # antall setninger

    def init_ngrams(self):
        """
        Initialiserer n-gramtabellen
        """
        self.ngrams[0] = 0 # totalt antall tagger
        for i in range(self.n):
            self.ngrams[i+1] = {}

    def count_ngrams(self, line):
        """
        Tar en setning som argument. Teller ngrammer (over tagger) og lagrer i ngramtabellen.
        """
        import re
        T2 = []
        t = re.compile(r'<(.+?):')
        T = t.findall(line) # plukk ut taggene, lagre i T
        for temptag in T: # for hver forelopige tagg
            t2 = re.compile(r'.+(\/|\|)(.*)') # let etter / eller |
            tag = t2.search(temptag) 
            if tag:
                T2.append(tag.group(2))
            else:
                T2.append(temptag)
        self.ngrams[0] = self.ngrams[0] + len(T) # oppdater totalt antall tagger
        for i in range(self.n): # for hver aktuelle ngramstorrelse
            self.count_ngram(T2, i+1) # teller ngrammer av storrelse i+1

    def count_ngram(self, T, nsize):
        """
        Tar ei liste med tagger (T) og en ngramstorrelse (nsize) som argumenter.
        Lagrer alle ngrammer av storrelse nsize.
        """
        length = len(T)
        for i in range(nsize - 1): 
            T.insert(0, None) # sett inn n-1 "dummy"-symboler
        for i in range(length): # for hver tagg
            ngram = [] # initialiser ngram
            for j in range(nsize): # for 0 til n-1
                ngram.append(T[i+j]) # legg til tagg til ngrammen
            ngram = tuple(ngram) # endre fra liste til tuppel
            if ngram in self.ngrams[nsize]: # hvis ngrammen er observert for
                self.ngrams[nsize][ngram] = self.ngrams[nsize][ngram] + 1 # oppdater antall
            else: # hvis ikke ngrammen er observert fr
                self.ngrams[nsize][ngram] = 1 # sett antall lik 1
        
    def count_words(self, L):
        """
        Tar en setning som argument (som liste). Teller ord/tagg-forekomster og
        suffiks/tagg-forekomster og lagrer i dictionaries
        """
        import re
        for wordtag in L: # for hvert ord/tagg-par
            w = re.compile(r'^(.+?)<')
            t = re.compile(r'<(.+?):')
            word = w.search(wordtag) # finn ordet
            tag = t.search(wordtag) # finn taggen
            t2 = re.compile(r'.+(\/|\|)(.*)') # let etter / eller |
            tag2 = t2.search(tag.group(1))
            if tag2:
                tagg = tag2.group(2)
            else:
                tagg = tag.group(1)
            self.count_word(word.group(1), tagg) # lagre ord/tagg-relasjon
            self.count_suffixes(word.group(1), tagg) # lagre suffiks/tagg-relasjoner

    def count_word(self, word, tag):
        """
        Tar et ord og en tagg som argumenter. Lagrer ordet og taggen i ordtabellen.
        """
        if word in self.words: # hvis ordet er sett fr
            if tag in self.words[word]: # hvis ordet er sett med den aktuelle taggen fr
                self.words[word][tag] = self.words[word][tag] + 1 # oppdater antall
            else:
                self.words[word][tag] = 1 # hvis ordet er sett fr, men ikke med den aktuelle taggen
        else:
            self.words[word] = {tag:1} # hvis ikke ordet er sett fr
        
    def count_suffixes(self, word, tag):
        """
        Tar et ord og en tagg som argumenter, lagrer ordets suffikser sammen med taggen i suffikstabellen
        """
        for i in range(1, self.s): # fra 1 til maks suffikslengde - 1...
            if len(word) > i:  #hvis ordet er minst like langt som den aktuelle suffikslengden
                suffix = word[-(i+1):] # plukk ut suffiks
                self.count_suffix(suffix,tag) # lagre suffiks/tagg-relasjon

    def count_suffix(self, suffix, tag):
        """
        Tar et suffiks og en tagg som argumenter, lagrer suffikset med taggen i suffikstabellen
        """
        if suffix in self.suffixes: # hvis suffikset er sett fr
            if tag in self.suffixes[suffix]: # hvis suffikset er sett med den aktuelle taggen
                self.suffixes[suffix][tag] = self.suffixes[suffix][tag] + 1 # oppdater antall
            else:
                self.suffixes[suffix][tag] = 1 # hvis suffikset er sett fr, men ikke med den aktuelle taggen
        else:
            self.suffixes[suffix] = {tag:1} # hvis ikke suffikset er sett fr

    def iterate_corpus(self):
        """
        Gr gjennom treningskorpus og lagrer n-grammer og tagg/ord-korrelasjoner
        """
        self.init_ngrams()
        import operator
        counter = 0
        corpus = open(self.data, 'r') # pne treningskorpus
        for line in corpus: # for hver setning
            self.sentences = self.sentences + 1
            L = line.split()
            self.count_ngrams(line) # tell n-grammer
            self.count_words(L) # tell ord og suffikser
            counter = counter + 1
            if operator.mod(counter,1000) == 0:
                print str(counter) + " linjer lest..."
        corpus.close()

            
class Print:
    """
    Klasse for skriving av antall til fil
    """
    def __init__(self, P):
        self.P = P # instans av klassen Count

    def print_ngrams(self):
        """
        Skriver ngrammer til fil
        """
        for i in self.P.ngrams:
            if i > 0:
                ngramfile = open(str(i)+ ".txt", 'w')
                for j in self.P.ngrams[i]:
                    ngramfile.write(str(j) + "\t" + str(self.P.ngrams[i][j]) + "\n")
                ngramfile.close()
                
        
    def print_words(self):
        """
        Skriver ord til fil
        """
        wordfile = open("words.txt", 'w')
        for word in self.P.words:
            for tag in self.P.words[word]:
                wordfile.write(str(word) + "\t" + str(tag) + "\t" + str(self.P.words[word][tag]) + "\n")
        wordfile.close()
                

    def print_suffixes(self):
        """
        Skriver suffikser til fil
        """
        suffixfile = open("suffixes.txt", 'w')
        for suffix in self.P.suffixes:
            for tag in self.P.suffixes[suffix]:
                suffixfile.write(str(suffix) + "\t" + str(tag) + "\t" + str(self.P.suffixes[suffix][tag]) + "\n")
        suffixfile.close()

    def print_to_files(self):
        """
        Skriver data til fil
        """
        print "Skriver til fil..." + "\n"
        self.print_ngrams()
        self.print_words()
        self.print_suffixes()
