# -*- coding: UTF-8 -*-
def flatten(L):
	flat = []
	for i in L:
		if (isinstance(i, list) or isinstance(i, tuple)):
			flat.extend(flatten(list(i)))
		else: 
			flat.append(i)
	return flat
	
def permute_candidates(L=[(1, 2, 3), ('x', 'y'), ('A', 'B')], 
					   make_copy=True):
	"""
	Produserer en liste med alle permutasjoner for kombinasjoner 
	av elementer i en liste med tupler.
	<n>=len(<L>) tupler i lista gir oss <n>-gram permutasjoner ut.
	"""
	import copy
	if make_copy:
		x = copy.deepcopy(L)
	else:
		x = L
	n = len(L)	#<n> kandidatsett gir oss <n>-gram permutasjoner
	while len(x) >= 2:
		cands1 = x.pop(0)
		cands2 = x.pop(0)
		temp = []
		for c1 in cands1:
			for c2 in cands2:
				temp.append([c1, c2])
		x.insert(0, temp)
	x = flatten(x)
	i = 0
	candidates = []
	while i < (len(x) - (n - 1)):
		candidates.append(tuple(x[i:i+n]))
		i += n
	#for et sett med unike n-gram, bruk "set"-operatoren:
	#candidates = set(candidates)
	return candidates


# Ola Huseth
# Dekoder for POS-tagging
# -*- coding: UTF-8 -*-

class ParB:
    """
    Klasse for parameter B - observasjonssannsynligheter
    """
    def __init__(self, source, Model):
        import copy
        self.source = copy.deepcopy(source) # kildesetning
        self.tab = {} # ord som nkler og dict. med kandidater som verdi
        self.Model = Model # hmm-modellen

    def create_tab(self):
        """
        Oppretter parameter B. 2-dimensjonal hash-tabell med
        ord/kandidatoversettelse som nkkel og interpolert log som verdi
        """
        count = 0 # ordteller
        for word in self.source: # for hvert input-ord
            self.tab[word] = self.Model.get_cand(word, count) # returnerer hash-tabell med kandidater/prob 
            count = count + 1
        self.tab[None] = {None : 0}
            
class ParA:
    """
    Klasse for parameter A - transisjonssannsynligheter
    """
    def __init__(self, source, Model, n=3):
        import copy
        self.source = copy.deepcopy(source) # kildesetning
        self.tab = {} # ngram (tuppel) som nkkel, interpolert log som verdi
        self.list = [] # liste med tuppelpar: "ekte" ngram og "abstrahert" ngram 
        self.Model = Model
        self.n = n # n-gramstrrelse

    def create_list(self, B):
        """
        Oppretter ei liste med alle n-grammer (tupler) som er aktuelle for dekoderen
        """
        length = len(self.source)
        for i in range(self.n-1):
            self.source.insert(0,None) # sett inn n-1 startsymboler
        for j in range(length): # for hver n-gram
            n_gram = self.source[j:(j+self.n)] # lagre n-gram over ord
            n_gram_list = [] # inneholder n tupler med kandidattaggene til ordene i n-grammen
            for k in n_gram: # for hvert ord i n-grammen
                cand = [] # kandidatliste for ordet
                for m in B.tab[k]: # for hver kandidattagg til ordet
                    cand.append(m) # legg til kandidaten i cand
                n_gram_list.append(tuple(cand)) # lagre kandidatlista som tuppel
            temp = permute_candidates(n_gram_list) # liste med tupler
            for tup in temp: # for hver tuppel
                self.list.append(tup) # legg til i self.list
        self.list = list(set(self.list)) # fjern dupliseringer
        ## 'permute_candidates' og 'flatten' er implementert av Kristian...

    def create_tab(self):
        """
        Oppretter parameter A. Hashtabell med ngrammer som nkler
        og interpolert log som verdi
        """
        import math
        for n_gram in self.list:
            self.tab[n_gram] = self.Model.get_trans_prob(n_gram)


class States:
    """
    Klasse for tilstandstabeller
    Hver tilstand er en (n-1)-tuppel
    """
    def __init__(self, source, n=3):
        import copy
        self.source = copy.deepcopy(source) # kildesetningen
        self.states = {} # tilst. som nkler, pos. som verdi 
        self.pos = [] # self.pos[x] angir tilst. i pos. x
        self.T = 0 # antall tilstander
        self.n = n #n-gramstrrelse

    def fill_tab(self, B):
        """
        Fyller tabellene self.pos og self.states
        """
        state_list = [] # liste for lagring av tilstander
        length = len(self.source)
        for i in range(self.n):
            self.source.insert(0,None) # sett inn n startsymboler
        for j in range(length + 2): # for hver tilstand
            n_gram = self.source[j:(j+self.n-1)] # lagre n-gram (lengde n-1) (utgangspunkt for tilstander)
            n_gram_list = [] # inneholder n tupler med kandidatoversettelsene til ordene i n-grammen
            for k in n_gram: # for hvert ord i n-grammen
                cand = [] # kandidatliste for ordet
                for m in B.tab[k]: # for hver kandidat til ordet
                    cand.append(m) # legg til kandidaten i cand
                n_gram_list.append(tuple(cand)) # lagre kandidatlista som tuppel
            temp = permute_candidates(n_gram_list) # liste med tupler
            for tup in temp: # for hver tuppel
                state_list.append(tup) # legg til i self.list
        state_list = set(state_list) # self.liste inneholder n mengden av tilstander 
        ## 'permute candidates' og 'flatten' implementert av Kristian...
        # lagring i tabellene 'pos' og 'states:
        for t in state_list: # for hver tilstand
            self.pos.append(t) # legg til i pos
            self.states[t] = self.T # legg til i states
            self.T = self.T+1 # oppdater tilstandstelleren
            


class Decoder:
    """
    Klasse for dekoderen.
    """
    def __init__(self, source, A, B, S, n=3):
        import copy
        self.source = copy.deepcopy(source)
        self.source.insert(0,None) ## sett inn startsymbol frst
        self.A = A.tab # parameter A
        self.B = B.tab # parameter B
        self.T = S.T # antall tilstander
        self.states = S.states # dict. med tilst. som nkkel, pos. som verdi
        self.pos = S.pos # liste hvor pos[x] gir tilstanden i posisjon x
        self.vtab = {} # viterbitabellen
        self.ptab = {} # pekertabellen
        self.last = []
        self.last2 = []
        self.n = n

    def score(self, i, j, k):
        """
        Regner ut verdien i en celle i viterbitabellen
        i = posisjonen til nværende kildeord
        j = kandidatordet man ser p (tekststreng)
        k = posisjonen til tilstanden man gr fra
        """
        state_list = list(self.pos[k]) # tilstanden man gr fra lagret som liste
        state_list.append(j) # legg til kandidatordet man ser p
        obs_prob = self.B[self.source[i]][j] # obs.prob. til kandidatordet
        last_prob = self.vtab[i-1][k] # verdi i cellen man gr fra
        transition_prob = self.A[tuple(state_list)] # transisjonssannsynlighet
        return last_prob + transition_prob + obs_prob # addisjon pga logaritmer

    def fill_first(self):
        """
        Fyller frste kolonne i viterbitabellen
        """
        temp = []
        for i in range(self.n-1):
            temp.append(None)
        self.vtab[0] = {self.states[tuple(temp)] : 0}
        self.last.append(self.states[tuple(temp)])

    def fill_rest(self):
        """
        Fyller viterbitabellen fra andre kolonne og ut
        """
        for i in range(1, len(self.source)): # for hvert ord
            self.last2 = self.last
            self.last = []
            self.vtab[i] = {} #initialiser kolonnen
            self.ptab[i] = {} #initialiser kolonnen
            for j in self.B[self.source[i]]: # for hvert kandidatord
                for k in self.last2: # for hver potensielle forrige tilstand
                    new_score = self.score(i, j, k) # finn ny score
                    state_list = list(self.pos[k]) # lag liste av forrige tilstand
                    state_list = state_list[1:] # fjern frste element
                    state_list.append(j) # hekt p kandidatord
                    state = self.states[tuple(state_list)] # posisjonen til tilstanden man ser p
                    if not state in self.vtab[i]: # hvis cellen ikke har ftt verdi
                        self.vtab[i][state] = new_score
                        self.ptab[i][state] = k
                    elif new_score > self.vtab[i][state]: # hvis ny_score er strre en den gamle verdien i cellen
                        self.vtab[i][state] = new_score
                        self.ptab[i][state] = k
            for m in range(0, self.T): # for hver tilstand gitt kildeord
                if m in self.vtab[i]: # hvis cellen har ftt verdi
                    self.last.append(m) # legg til tilstand i forrige

    def find_best_tags(self):
        """
        Finner beste verdi i siste kolonne av viterbitabellen og nster opp bakoverpekerne.
        Returnerer liste med de beste taggene.
        """
        val = -999999
        back_pointer = 0
        target = []
        # finn beste verdi i siste kolonne
        i = len(self.source)-1 # posisjonen til siste ord
        for j in range(0, self.T): # for hver tilstand
            if j in self.vtab[i]: # hvis cellen har ftt verdi
                if self.vtab[i][j] > val: # hvis verdien i cella er strre enn val
                    val = self.vtab[i][j] # oppdater val
                    back_pointer = self.ptab[i][j] # oppdater peker
                    state = list(self.pos[j])
                    word = state[self.n-2]
                    target = [word] # finn oversettelsen av siste ord
        # flg bakoverpekere
        for k in range(i-1, 0, -1):
            state = list(self.pos[back_pointer])
            word = state[self.n-2]
            target.insert(0, word) # legg til ordet i target
            back_pointer = self.ptab[k][back_pointer] # oppdater peker
        return target



######################################################################################################################

## 'source' er kildesteningen som liste
## 'n' er n-gramstrrelsen (trenger i utgangspunktet forskjellige Dekodere for forskjellige n...)        
## unigramtabell er enkel hashtabell med unigrammene (blir kanskje for stor til  ha i minnet?)
       
#B = ParB(sourcem, M)
#B.create_tab()
#A = ParA(source, M)
#A.create_list(B)
#A.create_tab()
#S = States(source)
#S.fill_tab(B)
#Viterbi = Decoder(source, A, B, S)
#Viterbi.fill_first()
#Viterbi.fill_rest()
#Viterbi.find_best_tags()



