package no.ntnu.idi.patternmatching.search.dictionary.medicalcodes.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import no.ntnu.idi.deid.documentunits.Token;
import no.ntnu.idi.deid.patternmatching.search.dictionary.medicalcodes.ICPC2_Codes;
import no.ntnu.idi.deid.searchresultunits.CategoryTag;
import no.ntnu.idi.deid.searchresultunits.Tag;

import org.junit.Before;
import org.junit.Test;

public class ICPC2_CodesTest {
	
	
	private String validCode;
	private String validCode2;
	private String validCode3;
	private String invalidCode;
	
	private String validDescription;
	private String validDescription2;
	private String validDescription3;
	private String validDescription3_2;
	
	private CategoryTag validTag;
	private CategoryTag validTag2;
	private CategoryTag validTag3;
	private CategoryTag invalidTag;
	
	private CategoryTag ExactValidTag;
	private CategoryTag ExactValidTag2;
	private CategoryTag ExactValidTag3;
	private CategoryTag ExactValidTag3_2;
	private CategoryTag ExactInvalidTag;
	
    @Before
    public void setUp() {
    	validCode = "A01";
    	validCode2 = "Z29";
    	validCode3 = "P75";
    	invalidCode = "A11HA0";
    	
    	validDescription = "Kronisk generell smerte";
    	validDescription2 = "Drikkevann forurensning";
    	validDescription3 = "Aerofagi psykogen";
    	validDescription3_2 = "Dissosiativ krampe";
    	
		validTag = ICPC2_Codes.searchICPC2Code(new Token(1, validCode));
		validTag2= ICPC2_Codes.searchICPC2Code(new Token(1, validCode2));
		validTag3 = ICPC2_Codes.searchICPC2Code(new Token(1, validCode3));
		invalidTag = ICPC2_Codes.searchICPC2Code(new Token(2, invalidCode));
		
		ExactValidTag = ICPC2_Codes.searchExactICPC2Description(validDescription);
		ExactValidTag2 = ICPC2_Codes.searchExactICPC2Description(validDescription2);
		ExactValidTag3 = ICPC2_Codes.searchExactICPC2Description(validDescription3);
		ExactValidTag3_2 = ICPC2_Codes.searchExactICPC2Description(validDescription3_2);
		ExactInvalidTag = ICPC2_Codes.searchExactICPC2Description("An invalid testString");
    }
		
	@Test
	public void testValidTag() {
		assertEquals(Tag.ICPC2, validTag.getTag());
		assertEquals(Tag.ICPC2, validTag2.getTag());
		assertEquals(Tag.ICPC2, validTag3.getTag());
	}
	
	@Test
	public void testValidCode() {
		assertEquals(validCode, validTag.getCode());
		assertEquals(validCode2, validTag2.getCode());
		assertEquals(validCode3, validTag3.getCode());
	}
	
	@Test
	public void testDescription() {	
		assertEquals(validDescription, validTag.getDescription());
		assertEquals(validDescription2, validTag2.getDescription());
		assertEquals(validDescription3, validTag3.getDescription());
	}
	
	@Test
	public void testSearchExactDescription() {	
		
		assertEquals(validDescription, ExactValidTag.getDescription());
		assertEquals(validDescription2, ExactValidTag2.getDescription());
		assertEquals(validDescription3, ExactValidTag3.getDescription());
		assertEquals(validDescription3_2, ExactValidTag3_2.getDescription());
	}
	
	@Test
	public void testSearchExactCode() {	
		assertEquals(validCode, ExactValidTag.getCode());
		assertEquals(validCode2, ExactValidTag2.getCode());
		assertEquals(validCode3, ExactValidTag3.getCode());
		assertEquals(validCode3, ExactValidTag3_2.getCode());
	}
	
	@Test
	public void testSearchExactTag() {	
		assertEquals(Tag.ICPC2, ExactValidTag.getTag());
		assertEquals(Tag.ICPC2, ExactValidTag2.getTag());
		assertEquals(Tag.ICPC2, ExactValidTag3.getTag());
		assertEquals(Tag.ICPC2, ExactValidTag3_2.getTag());
	}
	
	@Test
	public void testSearchExactInValid() {	
		assertNull(ExactInvalidTag);
	}
	
	@Test
	public void testInValidCode() {
		assertNull(invalidTag);
	}

}