package no.ntnu.idi.patternmatching.search.dictionary.medicalcodes.test;

import static org.junit.Assert.*;

import no.ntnu.idi.deid.documentunits.Token;
import no.ntnu.idi.deid.patternmatching.search.dictionary.medicalcodes.NCMP_Codes;
import no.ntnu.idi.deid.searchresultunits.CategoryTag;
import no.ntnu.idi.deid.searchresultunits.Tag;

import org.junit.Before;
import org.junit.Test;

public class NCMP_CodesTest {

	private String validCode;
	private String validCode2;
	private String validCode3;
	private String invalidCode;
	private Tag tag;
	
	private String validDescription;
	private String validDescription2;
	private String validDescription3;
	
	private CategoryTag validTag;
	private CategoryTag validTag2;
	private CategoryTag validTag3;
	private CategoryTag invalidTag;
	
	private CategoryTag ExactValidTag;
	private CategoryTag ExactValidTag2;
	private CategoryTag ExactValidTag3;
	private CategoryTag ExactInvalidTag;
	
    @Before
    public void setUp() {
    	tag = Tag.NCMP;
    	validCode = "AAAA00";
    	validCode2 = "ZWWB01";
    	validCode3 = "KAGD46";
    	invalidCode = "A11HA0";
    	
    	validDescription = "EEG overvåking med ekstrakranielle elektroder".toLowerCase();
    	validDescription2 = "Bruk av larynxmaske ved sedering og generell anestesi".toLowerCase();
    	validDescription3 = "Hemofiltrasjon".toLowerCase();
    	
		validTag = NCMP_Codes.searchNCMPCode(new Token(1, validCode));
		validTag2= NCMP_Codes.searchNCMPCode(new Token(2, validCode2));
		validTag3 = NCMP_Codes.searchNCMPCode(new Token(3, validCode3));
		invalidTag = NCMP_Codes.searchNCMPCode(new Token(4, invalidCode));
		
		ExactValidTag = NCMP_Codes.searchExactNCMPDescription(validDescription);
		ExactValidTag2 = NCMP_Codes.searchExactNCMPDescription(validDescription2);
		ExactValidTag3 = NCMP_Codes.searchExactNCMPDescription(validDescription3);
		ExactInvalidTag = NCMP_Codes.searchExactNCMPDescription("An invalid testString");
    }
		
	@Test
	public void testValidTag() {
		assertEquals(tag, validTag.getTag());
		assertEquals(tag, validTag2.getTag());
		assertEquals(tag, validTag3.getTag());
	}
	
	@Test
	public void testValidCode() {
		assertEquals(validCode.toLowerCase(), validTag.getCode());
		assertEquals(validCode2.toLowerCase(), validTag2.getCode());
		assertEquals(validCode3.toLowerCase(), validTag3.getCode());
	}
	
	@Test
	public void testDescription() {	
		assertEquals(validDescription, validTag.getDescription());
		assertEquals(validDescription2, validTag2.getDescription());
		assertEquals(validDescription3, validTag3.getDescription());
	}
	
	@Test
	public void testSearchExactDescription() {	
		assertEquals(validDescription, ExactValidTag.getDescription());
		assertEquals(validDescription2, ExactValidTag2.getDescription());
		assertEquals(validDescription3, ExactValidTag3.getDescription());
	}
	
	@Test
	public void testSearchExactCode() {	
		assertEquals(validCode.toLowerCase(), ExactValidTag.getCode());
		assertEquals(validCode2.toLowerCase(), ExactValidTag2.getCode());
		assertEquals(validCode3.toLowerCase(), ExactValidTag3.getCode());
	}
	
	@Test
	public void testSearchExactTag() {	
		assertEquals(tag, ExactValidTag.getTag());
		assertEquals(tag, ExactValidTag2.getTag());
		assertEquals(tag, ExactValidTag3.getTag());
	}
	
	@Test
	public void testSearchExactInValid() {	
		assertNull(ExactInvalidTag);
	}
	
	@Test
	public void testInValidCode() {
		assertNull(invalidTag);
	}

}
