package no.ntnu.idi.patternmatching.search.dictionary.medicalcodes.test;

import static org.junit.Assert.*;

import no.ntnu.idi.deid.documentunits.Token;
import no.ntnu.idi.deid.patternmatching.search.dictionary.medicalcodes.NCSP_Codes;
import no.ntnu.idi.deid.searchresultunits.CategoryTag;
import no.ntnu.idi.deid.searchresultunits.Tag;

import org.junit.Before;
import org.junit.Test;

public class NCSP_CodesTest {


	private String validCode;
	private String validCode2;
	private String validCode3;
	private String invalidCode;
	private Tag tag;
	
	private String validDescription;
	private String validDescription2;
	private String validDescription3;
	
	private CategoryTag validTag;
	private CategoryTag validTag2;
	private CategoryTag validTag3;
	private CategoryTag invalidTag;
	
	private CategoryTag ExactValidTag;
	private CategoryTag ExactValidTag2;
	private CategoryTag ExactValidTag3;
	private CategoryTag ExactInvalidTag;
	
    @Before
    public void setUp() {
    	tag = Tag.NCSP;
    	validCode = "AAA00";
    	validCode2 = "ZZU00";
    	validCode3 = "LEB00";
    	invalidCode = "A11HA022";
    	
    	validDescription = "Eksplorativ kraniotomi";
    	validDescription2 = "Fritt transplantat av slimhinne";
    	validDescription3 = "Biopsi av vagina";
    	
		validTag = NCSP_Codes.searchNCSPCode(new Token(1, validCode));
		validTag2= NCSP_Codes.searchNCSPCode(new Token(2, validCode2));
		validTag3 = NCSP_Codes.searchNCSPCode(new Token(3, validCode3));
		invalidTag = NCSP_Codes.searchNCSPCode(new Token(4, invalidCode));
		
		ExactValidTag = NCSP_Codes.searchExactDescription(validDescription);
		ExactValidTag2 = NCSP_Codes.searchExactDescription(validDescription2);
		ExactValidTag3 = NCSP_Codes.searchExactDescription(validDescription3);
		ExactInvalidTag = NCSP_Codes.searchExactDescription("An invalid testString");
    }
		
	@Test
	public void testValidTag() {
		assertEquals(tag, validTag.getTag());
		assertEquals(tag, validTag2.getTag());
		assertEquals(tag, validTag3.getTag());
	}
	
	@Test
	public void testValidCode() {
		assertEquals(validCode, validTag.getCode());
		assertEquals(validCode2, validTag2.getCode());
		assertEquals(validCode3, validTag3.getCode());
	}
	
	@Test
	public void testDescription() {	
		assertEquals(validDescription, validTag.getDescription());
		assertEquals(validDescription2, validTag2.getDescription());
		assertEquals(validDescription3, validTag3.getDescription());
	}
	
	@Test
	public void testSearchExactDescription() {	
		
		assertEquals(validDescription, ExactValidTag.getDescription());
		assertEquals(validDescription2, ExactValidTag2.getDescription());
		assertEquals(validDescription3, ExactValidTag3.getDescription());
	}
	
	@Test
	public void testSearchExactCode() {	
		assertEquals(validCode, ExactValidTag.getCode());
		assertEquals(validCode2, ExactValidTag2.getCode());
		assertEquals(validCode3, ExactValidTag3.getCode());
	}
	
	@Test
	public void testSearchExactTag() {	
		assertEquals(tag, ExactValidTag.getTag());
		assertEquals(tag, ExactValidTag2.getTag());
		assertEquals(tag, ExactValidTag3.getTag());
	}
	
	@Test
	public void testSearchExactInValid() {	
		assertNull(ExactInvalidTag);
	}
	
	@Test
	public void testInValidCode() {
		assertNull(invalidTag);
	}

}
