package no.ntnu.idi.patternmatching.search.dictionary.medicalcodes.test;

import static org.junit.Assert.*;

import no.ntnu.idi.deid.documentunits.Token;
import no.ntnu.idi.deid.patternmatching.search.dictionary.medicalcodes.Snomed_Elements;
import no.ntnu.idi.deid.searchresultunits.CategoryTag;
import no.ntnu.idi.deid.searchresultunits.Tag;

import org.junit.Before;
import org.junit.Test;

public class Snomed_ElementsTest {


	private String validCode;
	private String validCode2;
	private String validCode3;
	private String invalidCode;
	private Tag tag;
	
	private String validDescription;
	private String validDescription2;
	private String validDescription3;
	private String validDescription3_NOUNS;
	
	private CategoryTag validTag;
	private CategoryTag validTag2;
	private CategoryTag validTag3;
	private CategoryTag invalidTag;
	
	private CategoryTag ExactValidTag;
	private CategoryTag ExactValidTag2;
	private CategoryTag ExactValidTag3;
	private CategoryTag ExactInvalidTag;
	
    @Before
    public void setUp() {
    	tag = Tag.SNOMED;
    	validCode = "E00010";
    	validCode2 = "TY9910";
    	validCode3 = "TY9080";
    	invalidCode = "A11HA022";
    	
    	validDescription = "ukjent agens";
    	validDescription2 = "venstre underekstremitet og del av bekken";
    	validDescription3 = "begge underekstremiteter UNS";
    	validDescription3_NOUNS = "begge underekstremiteter";
    	
    	
		validTag = Snomed_Elements.searchInSnomedElements(new Token(1, validCode));
		validTag2= Snomed_Elements.searchInSnomedElements(new Token(2, validCode2));
		validTag3 = Snomed_Elements.searchInSnomedElements(new Token(3, validCode3));
		invalidTag = Snomed_Elements.searchInSnomedElements(new Token(4, invalidCode));
		
		ExactValidTag = Snomed_Elements.searchDescriptionWithoutUNS(validDescription);
		ExactValidTag2 = Snomed_Elements.searchDescriptionWithoutUNS(validDescription2);
		ExactValidTag3 = Snomed_Elements.searchDescriptionWithoutUNS(validDescription3_NOUNS);
		ExactInvalidTag = Snomed_Elements.searchDescriptionWithoutUNS("An invalid testString");
    }
		
	@Test
	public void testValidTag() {
		assertEquals(tag, validTag.getTag());
		assertEquals(tag, validTag2.getTag());
		assertEquals(tag, validTag3.getTag());
	}
	
	@Test
	public void testValidCode() {
		assertEquals(validCode, validTag.getCode());
		assertEquals(validCode2, validTag2.getCode());
		assertEquals(validCode3, validTag3.getCode());
	}
	
	@Test
	public void testDescription() {	
		assertEquals(validDescription, validTag.getDescription());
		assertEquals(validDescription2, validTag2.getDescription());
		assertEquals(validDescription3, validTag3.getDescription());
	}
	
	@Test
	public void testSearchExactDescription() {	
		
		assertEquals(validDescription, ExactValidTag.getDescription());
		assertEquals(validDescription2, ExactValidTag2.getDescription());
		assertEquals(validDescription3, ExactValidTag3.getDescription());
	}
	
	@Test
	public void testSearchExactCode() {	
		assertEquals(validCode, ExactValidTag.getCode());
		assertEquals(validCode2, ExactValidTag2.getCode());
		assertEquals(validCode3, ExactValidTag3.getCode());
	}
	
	@Test
	public void testSearchExactTag() {	
		assertEquals(tag, ExactValidTag.getTag());
		assertEquals(tag, ExactValidTag2.getTag());
		assertEquals(tag, ExactValidTag3.getTag());
	}
	
	@Test
	public void testSearchExactInValid() {	
		assertNull(ExactInvalidTag);
	}
	
	@Test
	public void testInValidCode() {
		assertNull(invalidTag);
	}


}
